try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO
import functools
import sys
from chaussette import logger


def hush(func):
    """Make the passed function silent."""
    @functools.wraps(func)
    def _silent(*args, **kw):
        old_stdout = sys.stdout
        old_stderr = sys.stderr
        sys.stdout = StringIO()
        sys.stderr = StringIO()
        debug = []

        def _debug(msg):
            debug.append(str(msg))

        old_debug = logger.debug
        logger.debug = _debug
        try:
            return func(*args, **kw)
        except:
            sys.stdout.seek(0)
            print(sys.stdout.read())
            sys.stderr.seek(0)
            print(sys.stderr.read())
            print('\n'.join(debug))
            raise
        finally:
            sys.stdout = old_stdout
            sys.stderr = old_stderr
            logger.debug = old_debug
    return _silent



import socket
from chaussette.util import create_socket
import bjoern


class Server(object):
    def __init__(self, listener, application=None, backlog=None,
                 socket_type=None, address_family=None):
        host, port = listener
        self.app = application
        self.sock = create_socket(host, port, address_family,
                                  socket_type, backlog=backlog)
        self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)

    def serve_forever(self):
        bjoern.server_run(self.sock, self.app)



import unittest
import os
import socket
import tempfile

from chaussette.util import create_socket, import_string


class TestUtil(unittest.TestCase):

    def test_create_socket(self):

        # testing various options

        # regular socket
        sock = create_socket('0.0.0.0')
        try:
            _, port = sock.getsockname()
            self.assertNotEqual(port, 0)
        finally:
            sock.close()

        # fd-based socket
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        sock.bind(('0.0.0.0', 0))
        _, port = sock.getsockname()

        sock2 = create_socket('fd://%d' % sock.fileno())
        try:
            _, port2 = sock2.getsockname()
            self.assertEqual(port2, port)
        finally:
            sock2.close()
            sock.close()

        # unix socket
        fd, path = tempfile.mkstemp()
        os.close(fd)
        sock = create_socket('unix://%s' % path)
        try:
            self.assertEqual('//' + path, sock.getsockname())
        finally:
            sock.close()
            os.remove(path)

    def test_import_string(self):
        self.assertRaises(ImportError, import_string, 'chaussette.calecon')
        imported = import_string('chaussette.tests.test_util.TestUtil')
        self.assertTrue(imported is TestUtil)



import os
import time
import sys
import socket
import threading
import select
import random
from six.moves import queue
import logging
import fcntl

import six


LOG_LEVELS = {
    "critical": logging.CRITICAL,
    "error": logging.ERROR,
    "warning": logging.WARNING,
    "info": logging.INFO,
    "debug": logging.DEBUG}

LOG_FMT = r"%(asctime)s [%(process)d] [%(levelname)s] %(message)s"
LOG_DATE_FMT = r"%Y-%m-%d %H:%M:%S"


def close_on_exec(fd):
    flags = fcntl.fcntl(fd, fcntl.F_GETFD)
    flags |= fcntl.FD_CLOEXEC
    fcntl.fcntl(fd, fcntl.F_SETFD, flags)


def configure_logger(logger, level='INFO', output="-"):
    loglevel = LOG_LEVELS.get(level.lower(), logging.INFO)
    logger.setLevel(loglevel)
    if output == "-":
        h = logging.StreamHandler()
    else:
        h = logging.FileHandler(output)
        close_on_exec(h.stream.fileno())
    fmt = logging.Formatter(LOG_FMT, LOG_DATE_FMT)
    h.setFormatter(fmt)
    logger.handlers = [h]


class ImportStringError(ImportError):
    """Provides information about a failed :func:`import_string` attempt."""

    #: String in dotted notation that failed to be imported.
    import_name = None
    #: Wrapped exception.
    exception = None

    def __init__(self, import_name, exception):
        self.import_name = import_name
        self.exception = exception

        msg = (
            'import_string() failed for %r. Possible reasons are:\n\n'
            '- missing __init__.py in a package;\n'
            '- package or module path not included in sys.path;\n'
            '- duplicated package or module name taking precedence in '
            'sys.path;\n'
            '- missing module, class, function or variable;\n\n'
            'Debugged import:\n\n%s\n\n'
            'Original exception:\n\n%s: %s')

        name = ''
        tracked = []
        for part in import_name.replace(':', '.').split('.'):
            name += (name and '.') + part
            imported = import_string(name, silent=True)
            if imported:
                tracked.append((name, getattr(imported, '__file__', None)))
            else:
                track = ['- %r found in %r.' % (n, i) for n, i in tracked]
                track.append('- %r not found.' % name)
                msg = msg % (import_name, '\n'.join(track),
                             exception.__class__.__name__, str(exception))
                break

        ImportError.__init__(self, msg)

    def __repr__(self):
        return '<%s(%r, %r)>' % (self.__class__.__name__, self.import_name,
                                 self.exception)


def import_string(import_name, silent=False):
    """Imports an object based on a string.  This is useful if you want to
    use import paths as endpoints or something similar.  An import path can
    be specified either in dotted notation (``xml.sax.saxutils.escape``)
    or with a colon as object delimiter (``xml.sax.saxutils:escape``).

    If `silent` is True the return value will be `None` if the import fails.

    For better debugging we recommend the new :func:`import_module`
    function to be used instead.

    :param import_name: the dotted name for the object to import.
    :param silent: if set to `True` import errors are ignored and
                   `None` is returned instead.
    :return: imported object
    """
    # force the import name to automatically convert to strings
    if not six.PY3 and isinstance(import_name, unicode):      # NOQA
        import_name = import_name.encode('utf-8')
    try:
        if ':' in import_name:
            module, obj = import_name.split(':', 1)
        elif '.' in import_name:
            module, obj = import_name.rsplit('.', 1)
        else:
            return __import__(import_name)
            # __import__ is not able to handle unicode strings in the fromlist
        # if the module is a package
        if not six.PY3 and isinstance(obj, unicode):        # NOQA
            obj = obj.encode('utf-8')
        try:
            return getattr(__import__(module, None, None, [obj]), obj)
        except (ImportError, AttributeError):
            # support importing modules not yet set up by the parent module
            # (or package for that matter)
            modname = module + '.' + obj
            __import__(modname)
            return sys.modules[modname]
    except ImportError as e:
        if not silent:
            six.reraise(ImportStringError, ImportStringError(import_name, e),
                        sys.exc_info()[2])


def hello_app(environ, start_response):
    status = '200 OK'
    headers = [('Content-type', 'text/plain')]
    start_response(status, headers)
    return [b'hello world']


_IN = _OUT = None
_DBS = queue.Queue()


_ITEMS = """\
<HTML>
  <BODY>
    Hello there.
  </BODY>
</HTML>""".split('\n')


class _FakeDBThread(threading.Thread):
    """Simulates a DB connection
    """
    def __init__(self):
        threading.Thread.__init__(self)
        self.read1, self.write1 = os.pipe()
        self.read2, self.write2 = os.pipe()
        self.running = False
        self.daemon = True

    def send_to_db(self, data):
        os.write(self.write1, data)

    def get_from_db(self):
        data = []
        while True:
            rl, __, __ = select.select([self.read2], [], [], 1.)
            if rl == []:
                print('nothing came back')
                continue
            current = os.read(self.read2, 1024)
            data.append(current)
            if current.strip().endswith('</HTML>'):
                break
        return data

    def run(self):
        self.running = True
        while self.running:
            rl, __, __ = select.select([self.read1], [], [], 1.)
            if rl == []:
                continue

            os.read(self.read1, 1024)

            for item in _ITEMS:
                os.write(self.write2, item + '\n')

    def stop(self):
        self.running = False
        self.join()
        for f in (self.read1, self.read2, self.write1, self.write2):
            os.close(f)


def setup_bench(config):
    # early patch
    if config.backend in ('gevent', 'fastgevent'):
        from gevent import monkey
        monkey.patch_all()
    elif config.backend == 'meinheld':
        from meinheld import patch
        patch.patch_all()

    # starting 10 threads in the background
    for i in range(10):
        th = _FakeDBThread()
        th.start()
        _DBS.put(th)

    time.sleep(0.2)


def teardown_bench(config):
    while not _DBS.empty():
        th = _DBS.get()
        th.stop()


_100BYTES = '*' * 100 + '\n'


def bench_app(environ, start_response):
    status = '200 OK'
    headers = [('Content-type', 'text/html')]
    start_response(status, headers)

    # math
    for i in range(10000):
        10 * 1000 * 1000

    duration = float(random.randint(25, 50) + 50)
    time.sleep(duration / 1000.)

    # I/O - sending 100 bytes, getting back an HTML page
    result = []

    # picking a DB
    db = _DBS.get(timeout=1.0)
    try:
        db.send_to_db(_100BYTES)
        for line in db.get_from_db():
            result.append(line)
    finally:
        _DBS.put(db)

    return result


def create_socket(host, port=0, family=socket.AF_INET, type=socket.SOCK_STREAM,
                  backlog=2048, blocking=True):
    if family == socket.AF_UNIX and not host.startswith('unix:'):
        raise ValueError('Your host needs to have the unix:/path form')
    if host.startswith('unix:') and family != socket.AF_UNIX:
        # forcing to unix socket family
        family = socket.AF_UNIX

    if host.startswith('fd://'):
        # just recreate the socket
        fd = int(host.split('://')[1])
        sock = socket.fromfd(fd, family, type)
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    else:
        sock = socket.socket(family, type)
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        if host.startswith('unix:'):
            filename = host[len('unix:'):]
            try:
                os.remove(filename)
            except OSError:
                pass
            sock.bind(filename)
        else:
            sock.bind((host, port))
        sock.listen(backlog)

    if blocking:
        sock.setblocking(1)
    else:
        sock.setblocking(0)
    return sock



try:
    import unittest2 as unittest
except ImportError:
    import unittest

import sys
from chaussette.backend import backends

IS_PYPY = hasattr(sys, 'pypy_version_info')

PY2 = ['bjoern', 'eventlet', 'fastgevent', 'gevent',
       'geventwebsocket', 'geventws4py', 'meinheld',
       'socketio', 'tornado', 'waitress',
       'wsgiref']
PYPY = ['tornado', 'waitress', 'wsgiref']
PY3 = ['meinheld', 'tornado', 'waitress', 'wsgiref']


class TestBackend(unittest.TestCase):

    def test_backends(self):
        _backends = backends()
        if sys.version_info[0] == 2:
            if IS_PYPY:
                expected = PYPY
            else:
                expected = PY2
        else:
            expected = PY3
        self.assertEqual(_backends, expected)



import socket
import tornado.ioloop
import tornado.netutil
import tornado.wsgi
from tornado.platform.auto import set_close_exec
from tornado.web import Application
from tornado.tcpserver import TCPServer
from tornado.httpserver import HTTPServer


class Server(object):
    def __init__(self, listener, application=None, backlog=2048,
                 socket_type=socket.SOCK_STREAM,
                 address_family=socket.AF_INET):
        self.address_family = address_family
        self.socket_type = socket_type
        host, port = listener

        if isinstance(application, Application):
            self._server = HTTPServer(application)
        elif isinstance(application, TCPServer):
            self._server = application
        elif callable(application):
            tapp = tornado.wsgi.WSGIContainer(application)
            self._server = HTTPServer(tapp)
        else:
            raise TypeError(
                "Unsupported application type: %r" % (application,))

        if host.startswith('fd://'):
            fd = int(host.split('://')[1])
            set_close_exec(fd)
            sock = socket.fromfd(fd, address_family, socket_type)
            sock.setblocking(0)
            socks = [sock]
        elif self.address_family == socket.AF_UNIX:
            filename = host[len('unix:'):]
            sock = tornado.netutil.bind_unix_socket(filename, backlog=backlog)
            socks = [sock]
        else:
            socks = tornado.netutil.bind_sockets(
                port, host, address_family, backlog)
        self._server.add_sockets(socks)
        self.application = application

    def serve_forever(self):
        tornado.ioloop.IOLoop.instance().start()



import socket
from wsgiref.simple_server import WSGIServer, WSGIRequestHandler
from six.moves import socketserver

from chaussette.util import create_socket


class ChaussetteHandler(WSGIRequestHandler):
    def __init__(self, sock, client_addr, server):
        if server.socket_type == socket.AF_UNIX:
            client_addr = ['0.0.0.0']
        WSGIRequestHandler.__init__(self, sock, client_addr, server)

    def address_string(self):
        if self.server.byfd or self.server.socket_type == socket.AF_UNIX:
            return '0.0.0.0'
        return WSGIRequestHandler.address_string(self)


class ChaussetteServer(WSGIServer):
    """WSGI Server that can reuse an existing open socket.
    """
    handler_class = ChaussetteHandler

    def __init__(self, server_address, app, bind_and_activate=True,
                 backlog=2048, socket_type=socket.SOCK_STREAM,
                 address_family=socket.AF_INET):
        self.address_family = address_family
        self.socket_type = socket_type
        socketserver.BaseServer.__init__(self, server_address,
                                         self.handler_class)
        self.set_app(app)

        host, port = self.server_address = server_address
        self.socket = create_socket(host, port,
                                    family=self.address_family,
                                    type=self.socket_type,
                                    backlog=backlog)
        self.byfd = host.startswith('fd://')
        if bind_and_activate:
            self.server_bind()
            self.server_activate()

    def server_activate(self):
        if self.byfd:
            return
        self.socket.listen(self.request_queue_size)

    def server_bind(self):
        if self.allow_reuse_address:
            self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        if not self.byfd:
            self.server_address = self.socket.getsockname()
            host, port = self.socket.getsockname()[:2]
            self.server_name = socket.getfqdn(host)
            self.server_port = port
        else:
            # XXX see how to get the fqnd with the fd
            self.server_name = self.server_address[0]
            self.server_port = self.server_address[1]

        self.setup_environ()



import socket
from waitress.server import WSGIServer


class Server(WSGIServer):
    address_family = socket.AF_INET
    socket_type = socket.SOCK_STREAM

    def __init__(self, listener, application=None, backlog=2048,
                 socket_type=socket.SOCK_STREAM,
                 address_family=socket.AF_INET):
        host, port = listener
        if host.startswith('fd://'):
            self._fd = int(host.split('://')[1])
        else:
            self._fd = None

        self._chaussette_family_and_type = address_family, socket_type
        super(Server, self).__init__(application, backlog=backlog, host=host,
                                     port=port)

    def create_socket(self, family, type):
        # Ignore parameters passed by waitress to use chaussette options
        family, type = self._chaussette_family_and_type

        self.family_and_type = family, type
        if self._fd is None:
            sock = socket.socket(family, type)
        else:
            sock = socket.fromfd(self._fd, family, type)
        sock.setblocking(0)
        self.set_socket(sock)

    def bind(self, listener):
        if self._fd is not None:
            return
        super(Server, self).bind(listener)

    def serve_forever(self):
        return self.run()



import os
from six.moves import configparser

from logging.config import fileConfig

try:
    from paste.deploy import loadapp
except ImportError:
    raise ImportError("You need to install PasteDeploy")


def paste_app(path):
    # Load the logging config from paste.deploy .ini, if any
    path, hsh, app = path.partition('#')

    parser = configparser.ConfigParser()
    parser.read([path])
    if parser.has_section('loggers'):
        config_file = os.path.abspath(path)
        fileConfig(
            config_file,
            dict(__file__=config_file, here=os.path.dirname(config_file))
        )

    return loadapp('config:%s%s%s' % (os.path.abspath(path), hsh, app))



import os
import socket
from meinheld import server


class Server(object):
    def __init__(self, listener, application=None, backlog=2048,
                 socket_type=socket.SOCK_STREAM,
                 address_family=socket.AF_INET):
        self.address_family = address_family
        self.socket_type = socket_type
        from meinheld import patch
        patch.patch_all()
        server.set_backlog(backlog)
        host, port = listener
        if host.startswith('fd://'):
            fd = int(host.split('://')[1])
            server.set_listen_socket(fd)
        else:
            if self.address_family == socket.AF_UNIX:
                filename = listener[0][len('unix:'):]

                try:
                    os.remove(filename)
                except OSError:
                    pass
                server.listen(filename)
            else:
                server.listen(listener)

        self.application = application

    def serve_forever(self):
        server.run(self.application)



from gevent import monkey

import socket
from gevent.pywsgi import WSGIServer, WSGIHandler
from chaussette.util import create_socket


class CustomWSGIHandler(WSGIHandler):
    def __init__(self, sock, address, server, rfile=None):
        if server.address_family == socket.AF_UNIX:
            address = ['0.0.0.0']
        WSGIHandler.__init__(self, sock, address, server, rfile)


class Server(WSGIServer):

    address_family = socket.AF_INET
    socket_type = socket.SOCK_STREAM
    handler_class = CustomWSGIHandler

    def __init__(self, listener, application=None, backlog=None,
                 spawn='default', log='default', handler_class=None,
                 environ=None, socket_type=None,
                 address_family=None, **ssl_args):
        monkey.noisy = False
        monkey.patch_all()
        if address_family:
            self.address_family = address_family
        if socket_type:
            self.socket_type = socket_type
        if handler_class:
            self.handler_class = handler_class

        host, port = listener
        self.socket = create_socket(host, port, self.address_family,
                                    self.socket_type, backlog=backlog)
        self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.server_address = self.socket.getsockname()
        super(Server, self).__init__(self.socket, application, None, spawn,
                                     log, self.handler_class, environ,
                                     **ssl_args)



try:
    import unittest2 as unittest
except ImportError:
    import unittest

import os
import sys

from tempfile import mkstemp


INI = """\
[composite:main]
use = egg:Paste#urlmap
/ = home

[app:home]
use = egg:Paste#static
document_root = %(here)s/htdocs
"""


@unittest.skipIf(sys.version_info[0] == 3, "Only Python 2")
class TestPasteApp(unittest.TestCase):

    def setUp(self):
        self.files = []

    def tearDown(self):
        for file_ in self.files:
            os.remove(file_)

    def _get_file(self):
        fd, path = mkstemp()
        os.close(fd)
        self.files.append(path)
        return path

    def test_app(self):
        from chaussette._paste import paste_app
        path = self._get_file()

        with open(path, 'w') as f:
            f.write(INI)

        app = paste_app(path)
        self.assertEqual(len(app), 1)



from gevent import monkey

import socket
from gevent.pywsgi import WSGIServer
from geventwebsocket.handler import WebSocketHandler
from chaussette.util import create_socket


class Server(WSGIServer):

    address_family = socket.AF_INET
    socket_type = socket.SOCK_STREAM

    def __init__(self, listener, application=None, backlog=None,
                 spawn='default', log='default', handler_class=None,
                 environ=None, socket_type=socket.SOCK_STREAM,
                 address_family=socket.AF_INET, **ssl_args):
        monkey.noisy = False
        monkey.patch_all()
        self.address_family = address_family
        self.socket_type = socket_type

        host, port = listener
        self.handler_class = WebSocketHandler
        self.socket = create_socket(host, port, self.address_family,
                                    self.socket_type, backlog=backlog)
        self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.server_address = self.socket.getsockname()
        super(Server, self).__init__(self.socket, application, None, spawn,
                                     log, handler_class, environ, **ssl_args)



import sys
import os
import argparse
import itertools
import socket

from chaussette import logger as chaussette_logger
from chaussette.util import import_string, configure_logger, LOG_LEVELS
from chaussette.backend import get, backends


def make_server(app, host=None, port=None, backend='wsgiref', backlog=2048,
                spawn=None, logger=None, address_family=socket.AF_INET,
                socket_type=socket.SOCK_STREAM):
    logger = logger or chaussette_logger
    logger.info('Application is %r' % app)
    if host.startswith('fd://') or host.startswith('unix:'):
        logger.info('Serving on %s' % host)
    else:
        logger.info('Serving on %s:%s' % (host, port))

    server_class = get(backend)
    logger.info('Using %r as a backend' % server_class)
    if spawn:
        logger.info('Spawning method: %r' % spawn)
    server_class_kwargs = {
        'backlog': backlog,
        'address_family': address_family,
        'socket_type': socket_type,
    }
    if spawn is not None:
        server_class_kwargs['spawn'] = spawn

    try:
        server = server_class((host, port), app, **server_class_kwargs)
    except TypeError:
        logger.exception("Failed to create backend %s, you might be trying "
                         "to use --spawn on a backend that does "
                         "not support it" % backend)
        raise
    return server


_ADDRESS_FAMILY = {
    'AF_UNIX': socket.AF_UNIX,
    'AF_INET': socket.AF_INET,
    'AF_INET6': socket.AF_INET6
}

_SOCKET_TYPE = {
    'SOCK_STREAM': socket.SOCK_STREAM,
    'SOCK_DGRAM': socket.SOCK_DGRAM,
    'SOCK_RAW': socket.SOCK_RAW,
    'SOCK_RDM': socket.SOCK_RDM,
    'SOCK_SEQPACKET': socket.SOCK_SEQPACKET
}

_NO_UNIX = ('waitress', 'fastgevent', 'eventlet')


def serve_paste(app, global_conf, **kw):
    port = int(kw.get('port', 8080))
    host = kw.get('host', '127.0.0.1')
    backlog = int(kw.get('backlog', 2048))
    backend = kw.get('backend', 'wsgiref')
    address_family = kw.get('address_family', 'AF_INET')
    address_family = _ADDRESS_FAMILY[address_family]
    socket_type = kw.get('socket_type', 'SOCK_STREAM')
    socket_type = _SOCKET_TYPE[socket_type]
    loglevel = kw.get('log_level', 'info')
    logoutput = kw.get('log_output', '-')
    logger = chaussette_logger
    configure_logger(logger, loglevel, logoutput)

    if address_family == socket.AF_UNIX and backend in _NO_UNIX:
        logger.info('Sorry %r does not support unix sockets' % backend)
        sys.exit(0)

    pre_hook = kw.get('pre_hook')
    if pre_hook is not None:
        pre_hook = import_string(pre_hook)
        logger.info('Running the pre-hook %r' % pre_hook)
        pre_hook(kw)

    post_hook = kw.get('post_hook')
    if post_hook is not None:
        post_hook = import_string(post_hook)

    try:
        httpd = make_server(app, host=host, port=port,
                            backend=backend, backlog=backlog,
                            logger=logger,
                            address_family=address_family,
                            socket_type=socket_type)
        try:
            httpd.serve_forever()
        except KeyboardInterrupt:
            sys.exit(0)
    finally:
        if post_hook is not None:
            logger.info('Running the post-hook %r' % post_hook)
            post_hook(kw)
    return 0


def main():
    sys.path.append(os.curdir)

    parser = argparse.ArgumentParser(description='Run some watchers.')
    parser.add_argument('--port', type=int, default=8080)
    parser.add_argument('--address-family', type=str, default='AF_INET',
                        choices=sorted(_ADDRESS_FAMILY.keys()))
    parser.add_argument('--socket-type', type=str, default='SOCK_STREAM',
                        choices=sorted(_SOCKET_TYPE.keys()))
    group = parser.add_mutually_exclusive_group()
    group.add_argument('--host', default='localhost')
    group.add_argument('--fd', type=int, default=-1)

    group.add_argument('--backlog', type=int, default=2048)
    parser.add_argument('--backend', type=str, default='wsgiref',
                        choices=backends())
    parser.add_argument('--use-reloader', action='store_true',
                        help="Restart server when source files change")
    parser.add_argument('--spawn', type=int, default=None,
                        help="Spawn type, only makes sense if the backend "
                             "supports it (gevent)")
    parser.add_argument('application', default='chaussette.util.hello_app',
                        nargs='?')
    parser.add_argument('arguments', default=[], nargs='*',
                        help="Optional arguments you may need for your app")
    parser.add_argument('--pre-hook', type=str, default=None)
    parser.add_argument('--post-hook', type=str, default=None)
    parser.add_argument('--python-path', type=str, default=None)

    log_levels = itertools.chain.from_iterable((key.upper(), key)
                                               for key in LOG_LEVELS.keys())
    parser.add_argument('--log-level', dest='loglevel', default='info',
                        choices=sorted(log_levels), help="log level")

    parser.add_argument('--log-output', dest='logoutput', default='-',
                        help="log output")
    args = parser.parse_args()

    application = args.application

    logger = chaussette_logger
    configure_logger(logger, args.loglevel, args.logoutput)

    if application.startswith('paste:'):
        from chaussette._paste import paste_app
        app = paste_app(application.split(':')[-1])
    else:
        app = import_string(application)

    if args.fd != -1:
        host = 'fd://%d' % args.fd
    else:
        host = args.host

    # pre-hook ?
    if args.pre_hook is not None:
        pre_hook = import_string(args.pre_hook)
        logger.info('Running the pre-hook %r' % pre_hook)
        pre_hook(args)

    # post-hook ?
    if args.post_hook is not None:
        post_hook = import_string(args.post_hook)
    else:
        post_hook = None

    address_family = _ADDRESS_FAMILY[args.address_family]
    if address_family == socket.AF_UNIX and args.backend in _NO_UNIX:
        logger.info('Sorry %r does not support unix sockets' % args.backend)
        sys.exit(0)

    def inner():
        try:
            httpd = make_server(app, host=host, port=args.port,
                                backend=args.backend, backlog=args.backlog,
                                spawn=args.spawn,
                                logger=logger,
                                address_family=address_family,
                                socket_type=_SOCKET_TYPE[args.socket_type])
            try:
                httpd.serve_forever()
            except KeyboardInterrupt:
                sys.exit(0)
        finally:
            if post_hook is not None:
                logger.info('Running the post-hook %r' % post_hook)
                post_hook(args)

    if args.use_reloader:
        try:
            from werkzeug.serving import run_with_reloader
        except ImportError:
            logger.info("Reloader requires Werkzeug: "
                        "'pip install werkzeug'")
            sys.exit(0)
        run_with_reloader(inner)
    else:
        inner()


if __name__ == '__main__':
    main()



import os
from gevent import monkey
monkey.patch_all()

from socketio import socketio_manage
from socketio.server import SocketIOServer
from socketio.namespace import BaseNamespace
from socketio.mixins import RoomsMixin, BroadcastMixin


class ChatNamespace(BaseNamespace, RoomsMixin, BroadcastMixin):
    def on_nickname(self, nickname):
        self.request['nicknames'].append(nickname)
        self.socket.session['nickname'] = nickname
        self.broadcast_event('announcement', '%s has connected' % nickname)
        self.broadcast_event('nicknames', self.request['nicknames'])
        # Just have them join a default-named room
        self.join('main_room')

    def recv_disconnect(self):
        # Remove nickname from the list.
        nickname = self.socket.session['nickname']
        self.request['nicknames'].remove(nickname)
        self.broadcast_event('announcement', '%s has disconnected' % nickname)
        self.broadcast_event('nicknames', self.request['nicknames'])

        self.disconnect(silent=True)

    def on_user_message(self, msg):
        self.emit_to_room('main_room', 'msg_to_room',
                          self.socket.session['nickname'], msg)

    def recv_message(self, message):
        print("PING!!!", message)


class Application(object):
    def __init__(self):
        self.buffer = []
        # Dummy request object to maintain state between Namespace
        # initialization.
        self.request = {
            'nicknames': [],
        }

    def __call__(self, environ, start_response):
        path = environ['PATH_INFO'].strip('/')

        if not path:
            start_response('200 OK', [('Content-Type', 'text/html')])
            return ['<h1>Welcome. '
                    'Try the <a href="/chat.html">chat</a> example.</h1>']

        root = os.path.dirname(__file__)
        if path.startswith('static/') or path == 'chat.html':
            path = os.path.join(root, path)
            try:
                data = open(path).read()
            except Exception:
                return not_found(start_response)

            if path.endswith(".js"):
                content_type = "text/javascript"
            elif path.endswith(".css"):
                content_type = "text/css"
            elif path.endswith(".swf"):
                content_type = "application/x-shockwave-flash"
            else:
                content_type = "text/html"

            start_response('200 OK', [('Content-Type', content_type)])
            return [data]

        if path.startswith("socket.io"):
            socketio_manage(environ, {'': ChatNamespace}, self.request)
        else:
            return not_found(start_response)


_app = Application()


def app(environ, start_response):
    return _app(environ, start_response)


def not_found(start_response):
    start_response('404 Not Found', [])
    return ['<h1>Not Found</h1>']


if __name__ == '__main__':
    print('Listening on port 8080 and on port 843 (flash policy server)')
    SocketIOServer(('0.0.0.0', 8080), Application(),
                   resource="socket.io", policy_server=True,
                   policy_listener=('0.0.0.0', 10843)).serve_forever()



import sys
from chaussette.backend import _wsgiref


_backends = {'wsgiref': _wsgiref.ChaussetteServer}

try:
    from chaussette.backend import _waitress
    _backends['waitress'] = _waitress.Server
except ImportError:
    pass

try:
    from chaussette.backend import _meinheld
    _backends['meinheld'] = _meinheld.Server
except ImportError:
    pass

try:
    from chaussette.backend import _tornado
    _backends['tornado'] = _tornado.Server
except ImportError:
    pass


PY3 = sys.version_info[0] == 3

if not PY3:
    try:
        from chaussette.backend import _gevent
        _backends['gevent'] = _gevent.Server

        from chaussette.backend import _fastgevent
        _backends['fastgevent'] = _fastgevent.Server
    except ImportError:
        pass

    try:
        from chaussette.backend import _geventwebsocket
        _backends['geventwebsocket'] = _geventwebsocket.Server
    except ImportError:
        pass

    try:
        from chaussette.backend import _geventws4py
        _backends['geventws4py'] = _geventws4py.Server
    except ImportError:
        pass

    try:
        from chaussette.backend import _eventlet
        _backends['eventlet'] = _eventlet.Server
    except ImportError:
        pass

    try:
        from chaussette.backend import _socketio
        _backends['socketio'] = _socketio.Server
    except ImportError:
        pass

    try:
        from chaussette.backend import _bjoern
        _backends['bjoern'] = _bjoern.Server
    except ImportError:
        pass


def register(name, server):
    _backends[name] = server


def get(name):
    return _backends[name]


def backends():
    return sorted(_backends.keys())



import sys
from setuptools import setup, find_packages
from chaussette import __version__


if (not hasattr(sys, 'version_info')
        or sys.version_info < (2, 6, 0, 'final')
        or (sys.version_info > (3,)
            and sys.version_info < (3, 3, 0, 'final'))):
    raise SystemExit("Chaussette requires Python 2.6, 2.7, 3.3 or later.")

PYPY = hasattr(sys, 'pypy_version_info')

install_requires = ['six >= 1.3.0']

try:
    import argparse     # NOQA
except ImportError:
    install_requires.append('argparse')


with open('README.rst') as f:
    README = f.read()


tests_require = ['nose', 'waitress', 'tornado',
                 'requests', 'minimock']

if not PYPY:
    tests_require += ['meinheld', 'greenlet']

if sys.version_info[0] == 2:
    tests_require += ['PasteDeploy', 'Paste', 'unittest2', 'ws4py']
    if not PYPY:
        tests_require += ['gevent', 'gevent-websocket', 'eventlet',
                          'gevent-socketio']


setup(name='chaussette',
      version=__version__,
      url='http://chaussette.readthedocs.org',
      packages=find_packages(),
      description=("A WSGI Server for Circus"),
      long_description=README,
      author="Mozilla Foundation & Contributors",
      author_email="services-dev@lists.mozila.org",
      include_package_data=True,
      zip_safe=False,
      classifiers=[
          "Programming Language :: Python",
          "Programming Language :: Python :: 2",
          "Programming Language :: Python :: 2.6",
          "Programming Language :: Python :: 2.7",
          "Programming Language :: Python :: 3",
          "Programming Language :: Python :: 3.3",
          "Programming Language :: Python :: 3.4",
          "License :: OSI Approved :: Apache Software License",
          "Development Status :: 3 - Alpha"],
      install_requires=install_requires,
      tests_require=tests_require,
      test_suite='nose.collector',
      entry_points="""
      [console_scripts]
      chaussette = chaussette.server:main

      [paste.server_runner]
      main = chaussette.server:serve_paste
      """)



# coding=utf-8
"""
Tests for server.py
"""
import subprocess
import sys
import time
try:
    import unittest2 as unittest
except ImportError:
    import unittest

import minimock
import requests
import socket

from chaussette.backend import backends
import chaussette.server
from chaussette.util import configure_logger
from chaussette import logger
from chaussette.tests.support import hush


@unittest.skipIf(sys.version_info[0] == 3, "Not py3")
class TestServer(unittest.TestCase):
    """
    Test server.py
    """

    def setUp(self):
        """
        Setup
        :return:
        """
        super(TestServer, self).setUp()
        configure_logger(logger, 'CRITICAL')

        self.tt = minimock.TraceTracker()
        try:
            self.old = socket.socket.bind
            socket.socket.bind = lambda x, y: None
        except AttributeError:
            self.old = None

    def tearDown(self):
        """
        tearDown
        :return:
        """
        super(TestServer, self).tearDown()
        minimock.restore()
        if self.old is not None:
            socket.socket.bind = self.old

    def test_make_server(self):
        """
        Test all backends with default params
        :return:
        """

        # nose does not have great support for parameterized tests
        for backend in backends():
            self.tt = minimock.TraceTracker()
            self._check_make_server(backend)
            minimock.restore()

    def _check_make_server(self, backend):
        mocked_backend = minimock.Mock('Backend', returns='backend_impl',
                                       tracker=self.tt)
        minimock.mock('chaussette.server.get', returns=mocked_backend,
                      tracker=self.tt)
        server = chaussette.server.make_server('app', 'host', 'port',
                                               backend)
        minimock.assert_same_trace(self.tt, '\n'.join([
            "Called chaussette.server.get('%s')" % backend,
            "Called Backend(",
            "    ('host', 'port'),",
            "   'app',",
            "   address_family=2,",
            "   backlog=2048,",
            "socket_type=1)"
        ]))
        self.assertEqual(server, 'backend_impl')

    def test_make_server_spawn(self):
        """
        Check the spawn option for the backend that support it
        :return:
        """
        for backend in ['gevent', 'fastgevent', 'geventwebsocket',
                        'socketio']:
            self.tt = minimock.TraceTracker()
            self._check_make_server_spawn(backend)
            minimock.restore()

    def _check_make_server_spawn(self, backend):
        mocked_backend = minimock.Mock('Backend', returns='backend_impl',
                                       tracker=self.tt)
        minimock.mock('chaussette.server.get', returns=mocked_backend,
                      tracker=self.tt)
        server = chaussette.server.make_server('app', 'host', 'port',
                                               backend, spawn=5)
        minimock.assert_same_trace(self.tt, '\n'.join([
            "Called chaussette.server.get('%s')" % backend,
            "Called Backend(",
            "    ('host', 'port'),",
            "   'app',",
            "   address_family=2,",
            "   backlog=2048,",
            "   socket_type=1,",
            "   spawn=5)"
        ]))
        self.assertEqual(server, 'backend_impl')

    def test_make_server_spawn_fail(self):
        """
        Check the spawn option for a backend that does not support it
        :return:
        """
        self.assertRaises(TypeError, chaussette.server.make_server, 'app',
                          'host', 'port', spawn=5)


class TestMain(unittest.TestCase):
    """
    Test server.py
    """

    def setUp(self):
        super(TestMain, self).setUp()
        self.argv = list(sys.argv)
        configure_logger(logger, 'CRITICAL')

    def tearDown(self):
        super(TestMain, self).tearDown()
        sys.argv[:] = self.argv

    def _launch(self, backend):
        cmd = '%s -m chaussette.server --backend %s'
        cmd = cmd % (sys.executable, backend)
        print(cmd)
        proc = subprocess.Popen(cmd.split(), stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE)
        for _ in range(10):
            time.sleep(0.2)
            try:
                s = socket.create_connection(('localhost', 8080), 1)
                s.close()
                break
            except socket.error:
                continue
        return proc

    @hush
    def test_main(self):
        for backend in backends():
            resp = None
            server = self._launch(backend)
            try:
                # socketio is not a WSGI Server.
                # So we check only it can be started.
                if backend == 'socketio':
                    continue
                resp = requests.get('http://localhost:8080')
                status = resp.status_code
                self.assertEqual(status, 200, backend)
                self.assertEqual(resp.text, u"hello world")
            finally:
                server.terminate()
                if resp is not None:
                    resp.connection.close()



import socket
import eventlet
from eventlet import wsgi
from chaussette.util import create_socket


class Server(object):
    address_family = socket.AF_INET
    socket_type = socket.SOCK_STREAM

    def __init__(self, listener, application=None, backlog=2048,
                 socket_type=socket.SOCK_STREAM,
                 address_family=socket.AF_INET):
        self.address_family = address_family
        self.socket_type = socket_type
        eventlet.monkey_patch()
        host, port = listener
        self.socket = create_socket(host, port, self.address_family,
                                    self.socket_type, backlog=backlog)
        self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.application = application

    def serve_forever(self):
        wsgi.server(self.socket, self.application)



import socket

from chaussette.util import create_socket

from socketio.server import SocketIOServer
from socketio.handler import SocketIOHandler
from socketio.policyserver import FlashPolicyServer


class _SocketIOHandler(SocketIOHandler):
    def __init__(self, config, sock, address, server, rfile=None):
        if server.socket_type == socket.AF_UNIX:
            address = ['0.0.0.0']
        SocketIOHandler.__init__(self, config, sock, address, server, rfile)


class Server(SocketIOServer):

    address_family = socket.AF_INET
    socket_type = socket.SOCK_STREAM
    handler_class = _SocketIOHandler

    def __init__(self, *args, **kwargs):
        address_family = kwargs.pop('address_family', socket.AF_INET)
        socket_type = kwargs.pop('socket_type', socket.SOCK_STREAM)
        backlog = kwargs.pop('backlog', 2048)

        listener = args[0]
        if isinstance(listener, tuple):
            host, port = listener
            _socket = create_socket(host, port, address_family,
                                    socket_type, backlog=backlog)
            _socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            args = [_socket] + list(args[1:])
        else:
            # it's already a socket..
            host, port = listener.getsockname()

        # socketio makes the assumption that listener is a host/port
        # tuple, which is a false assumption, it can be a socket.
        # it uses listener in its constructor to set the policy server
        #
        # Let's set it ourselves here afterwards.
        old_policy_server = kwargs.pop('policy_server', True)
        kwargs['policy_server'] = False
        super(Server, self).__init__(*args, **kwargs)

        if old_policy_server:
            policylistener = kwargs.pop('policy_listener', (host, 10843))
            self.policy_server = FlashPolicyServer(policylistener)



from ws4py.server.geventserver import (WebSocketWSGIHandler,
                                       GEventWebSocketPool)
from chaussette.backend._gevent import Server as GeventServer


class Server(GeventServer):
    handler_class = WebSocketWSGIHandler

    def __init__(self, *args, **kwargs):
        GeventServer.__init__(self, *args, **kwargs)
        self.pool = GEventWebSocketPool()

    def stop(self, *args, **kwargs):
        self.pool.clear()
        self.pool = None
        GeventServer.stop(self, *args, **kwargs)
